@@ -169,7 +169,8 @@ class DBMS:
 				mt_records = await t.history(limit=1024).flatten()
 				for record in mt_records:
 					if name.lower() == record.content.split(chr(0x2502))[0]:
-						headers = self.build_table_headers(record.content)
+						headers = self.build_table_headers(record)
+						del headers[0] # Don't track id here
 						header_row = record
 						break
 			if t.name.lower() == name.lower():
@@ -278,7 +279,7 @@ class DBMS:
 				mt_records = await t.history(limit=1024).flatten()
 				for record in mt_records:
 					if against.lower() == record.content.split(chr(0x2502))[0]:
-						headers = self.build_table_headers(record.content)
+						headers = self.build_table_headers(record)
 						break
 			if t.name.lower() == against.lower():
 				table = t
@@ -353,7 +354,8 @@ class DBMS:
 				mt_records = await t.history(limit=1024).flatten()
 				for record in mt_records:
 					if against.lower() == record.content.split(chr(0x2502))[0].lower():
-						headers = self.build_table_headers(record.content)
+						headers = self.build_table_headers(record)
+						del headers[0] # Don't track id here
 						break
 			if t.name.lower() == against.lower():
 				table = t
@@ -407,7 +409,7 @@ class DBMS:
 				mt_records = await t.history(limit=1024).flatten()
 				for record in mt_records:
 					if against.lower() == record.content.split(chr(0x2502))[0]:
-						headers = self.build_table_headers(record.content)
+						headers = self.build_table_headers(record)
 						break
 			if t.name.lower() == against.lower():
 				table = t
@@ -427,8 +429,9 @@ class DBMS:
 			tr = TableRow(headers)
 			split_rows = raw.content.split(chr(0x2502))
 			del split_rows[len(split_rows)-1]
+			tr.update_record(0, str(raw.id)) # Primary key
 			for i in range(len(split_rows)):
-				tr.update_record(i, split_rows[i])
+				tr.update_record(i+1, split_rows[i])
 			rows.append(tr)
 		
 		clauses = self.parse_where(where)
@@ -450,8 +453,7 @@ class DBMS:
 
 		for i in range(len(rows)):
 			if rows[i] is not None:
-				#print(str(rows[i])) #
-				await raw_rows[i].edit(content=str(rows[i]))
+				await raw_rows[i].edit(content=rows[i].writable())
 
 		# cleanup
 		if adstore is not None:
@@ -477,7 +479,7 @@ class DBMS:
 				mt_records = await t.history(limit=1024).flatten()
 				for record in mt_records:
 					if against.lower() == record.content.split(chr(0x2502))[0]:
-						headers = self.build_table_headers(record.content)
+						headers = self.build_table_headers(record)
 						break
 			if t.name.lower() == against.lower():
 				table = t
@@ -493,8 +495,9 @@ class DBMS:
 			tr = TableRow(headers)
 			split_rows = raw.content.split(chr(0x2502))
 			del split_rows[len(split_rows)-1]
+			tr.update_record(0, str(raw.id)) # Primary key
 			for i in range(len(split_rows)):
-				tr.update_record(i, split_rows[i])
+				tr.update_record(i+1, split_rows[i])
 			rows.append(tr)
 		
 		clauses = self.parse_where(where)
@@ -792,7 +795,7 @@ class DBMS:
 			for substr in checkstr.split(" "):
 				if not substr.isalnum():
 					return True
-				if any(illegals == substr.lower() for illegals in ["select", "from", "against", "where", "use", "create", "alter", "drop", "delete", "and", "or", "in"]):
+				if any(illegals == substr.lower() for illegals in ["select", "from", "against", "where", "use", "create", "alter", "drop", "delete", "and", "or", "in", "id"]):
 					return True
 		return False
 
@@ -808,8 +811,9 @@ class DBMS:
 		return True
 
 	def build_table_headers(self, stream):
-		arr = stream.split(chr(0x2502))
+		arr = stream.content.split(chr(0x2502))
 		headers = []
+		headers.append(TableHeader("id int", True)) # Message ID = Primary key
 		for i in range(len(arr)):
 			if i != 0:
 				headers.append(TableHeader(arr[i]))
@@ -898,10 +902,13 @@ class TableRow:
 		if records is not None:
 			records_raw = records.content.split(chr(0x2502))
 			del records_raw[len(records_raw)-1]
-			if not len(records_raw) == len(self.headers):
+			if not len(records_raw) + 1 == len(self.headers):
 				raise Exception("Number of records do not match expected headers")
 			for i in range(len(self.headers)):
-				self.records.append(TableRecord(self.headers[i], records_raw[i]))
+				if i == 0: # Primary key
+					self.records.append(TableRecord(self.headers[i], str(records.id)))
+				else:
+					self.records.append(TableRecord(self.headers[i], records_raw[i-1]))
 		elif table_records is not None:
 			if not len(table_records) == len(self.headers):
 				raise Exception("Number of records do not match expected headers")
@@ -930,10 +937,17 @@ class TableRow:
 			raise TypeError("index must be an int")
 		if index > len(self.headers) or index < 0:
 			raise IndexError("index out of bounds")
-		if self.headers[index].is_primary_key == True:
-			raise Exception("Cannot update primary key")
 		self.records[index] = TableRecord(self.headers[index], data.strip())
 
+	def writable(self):
+		"""String of TableRow excluding id for writing to the database"""
+		rs = ""
+		for i in range(len(self.records)):
+			if i == 0:
+				continue
+			rs += str(self.records[i].data) + chr(0x2502)
+		return rs
+
 class TableRecord:
 	def __init__(self, datatype, data):
 		self.datatype = datatype
